package de.desy.acop.video.displayer;

/**
 * Image flag enumeration
 * 
 * @author mdavid
 * 
 */
public enum ImageFlag {
	/**
	 * if set, appended video data is ordered little endian (on video format
	 * types where this is applicable)
	 */
	LITTLE_ENDIAN_BYTE_ORDER(0x1),

	/**
	 * If set, original image data (first digital image data) can be restored
	 * out of the appended image bits
	 */
	IMAGE_LOSSLESS(0x2),

	/**
	 * If set, image was flipped around horizontal axis on its way
	 */
	HORIZONTAL_FLIP_180(0x4),

	/**
	 * If set, image was flipped around vertical axis on its way
	 */
	VERTICAL_FLIP_180(0x8);

	private static final String LITTLEENDIAN = "LITTLEENDIAN";
	private static final String BIGENDIAN = "BIGENDIAN";
	private static final String LOSSLESS = " LOSSLESS";
	private static final String LOSSY = " LOSSY";
	private static final String HORIZONTALFLIP180 = " HORIZONTAL_FLIP_180";
	private static final String VERTICALFLIP180 = " VERTICAL_FLIP_180";

	private final int id;

	ImageFlag(int id) {
		this.id = id;
	}

	public int getId() {
		return id;
	}

	/**
	 * Decodes flags integer into human readable string format
	 * 
	 * @param flags
	 * @return string representation of the passed aFlags
	 *         (frameHeader.imageFlags)
	 */
	public static String toString(int flags) {
		StringBuilder sb = new StringBuilder();

		if ((flags & ImageFlag.LITTLE_ENDIAN_BYTE_ORDER.getId()) != 0)
			sb.append(LITTLEENDIAN);
		else
			sb.append(BIGENDIAN);

		if ((flags & ImageFlag.IMAGE_LOSSLESS.getId()) != 0)
			sb.append(LOSSLESS);
		else
			sb.append(LOSSY);

		if ((flags & ImageFlag.HORIZONTAL_FLIP_180.getId()) != 0)
			sb.append(HORIZONTALFLIP180);

		if ((flags & ImageFlag.VERTICAL_FLIP_180.getId()) != 0)
			sb.append(VERTICALFLIP180);

		return sb.toString();
	}
}
