/*
 * Created on Aug 24, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.server.devices;

//import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Set;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.server.equipment.TEquipmentModule;
import de.desy.tine.server.logger.MsgLog;

public class TDeviceList
{
  private final static int TDeviceListCapacity = 256;
  private LinkedHashMap<String, TDevice> myDeviceData = new LinkedHashMap<String, TDevice>(TDeviceListCapacity);
  private LinkedList<String> myDeviceNames = new LinkedList<String>();
  private int lastRegisteredDevice = -1;
  private boolean propertyOriented = false;
  public void setPropertyOriented(boolean value) { propertyOriented = value; }
  public boolean isPropertyOriented() { return propertyOriented; }
  public int getLastRegisteredDevice() { return lastRegisteredDevice; }
  public int getNumberOfDevices() { return myDeviceNames.size(); }
  public void refreshAlarms()
  {
    TDevice[] dlst = getDeviceList();
    if (dlst == null) return;
    
    for (int i=0; i<dlst.length; i++)
    {
      dlst[i].processAlarms();
    }
    return;        
  }
  public void clearAlarms()
  {
    TDevice[] dlst = getDeviceList();
    if (dlst == null) return;
    for (int i=0; i<dlst.length; i++)
    {
      dlst[i].clearAlarms();
    }
    return;
  }
  public void removeAllAlarms()
  {
    TDevice[] dlst = getDeviceList();
    if (dlst == null) return;
    for (int i=0; i<dlst.length; i++)
    {
      dlst[i].removeAlarms();
    }
  }
  public void touchActiveAlarms()
  {
    int thisTimeStamp = (int)(System.currentTimeMillis()/1000);
    touchActiveAlarms(thisTimeStamp);
  }
  public void touchActiveAlarms(int timestamp)
  {
    TDevice[] dlst = getDeviceList();
    if (dlst == null) return;
    for (int i=0; i<dlst.length; i++)
    {
      dlst[i].touchAlarms(timestamp);
    }    
  }
  public Set getDeviceSet()
  {
    return myDeviceData.entrySet();
  }
  public String[] getDeviceNameList()
  {
    return (String[])myDeviceNames.toArray(new String[0]);
  }
  public TDevice[] getDeviceList()
  {
    return (TDevice[])myDeviceData.values().toArray(new TDevice[0]);
  }
  public LinkedList<String> getDeviceLinkedList() { return myDeviceNames; }
  public TDevice getDevice(String devName)
  {
    if (devName == null) return null;
    if (devName.startsWith("#"))
    {
      try 
      {
        return getDevice(Integer.parseInt((devName.substring(1))));
      }
      catch (Exception e)
      {
        MsgLog.log("getDevice",e.toString(),TErrorList.code_failure,e,0);                          
        return null;
      }
    }
    String devNameUC = devName.toUpperCase();
    if (!myDeviceData.containsKey(devNameUC)) return null;
    return (TDevice)myDeviceData.get(devNameUC);
  }
  public TDevice getDevice(int devNumber)
  {
    if (devNumber < 0 || devNumber >= myDeviceNames.size()) return null;
    String devName = (String)myDeviceNames.get(devNumber);
    return (TDevice)myDeviceData.get(devName.toUpperCase());
  }
  public void renameDevice(String oldName,String newName)
  {
    TDevice tdv = getDevice(oldName);
    if (tdv == null) return;
    tdv.setName(newName);
    int dv = tdv.devNumber;
    myDeviceData.remove(oldName.toUpperCase());
    myDeviceData.put(newName.toUpperCase(),tdv);
    myDeviceNames.set(dv,newName);
  }
  public void addDevice(String devName,int devNumber,TEquipmentModule eqm)
  {
    if (devNumber < 0) return;
    if (devNumber < myDeviceNames.size())
    {
      myDeviceNames.set(devNumber,devName);
      return;
    }
    else
    {
      if (devNumber > lastRegisteredDevice + 1)
      {
        for (int i=lastRegisteredDevice+1; i<devNumber; i++) myDeviceNames.add("#"+i);
      }
      lastRegisteredDevice = devNumber;
      myDeviceData.put(devName.toUpperCase(),new TDevice(devName,devNumber,eqm));
      myDeviceNames.add(devName);
    }
  }
  public void addDevice(String devName,TEquipmentModule eqm)
  {
    addDevice(devName,lastRegisteredDevice + 1,eqm);
  }
  public void addDevice(TDevice device)
  {
    if (device == null) return;
    myDeviceData.put(device.devName.toUpperCase(),device);
    myDeviceNames.add(device.devName);
  }
  public int getDeviceNumber(String devName)
  {
    if (devName.startsWith("#")) 
    {
      try 
      {
        return Integer.parseInt((devName.substring(1)));
      }
      catch (NumberFormatException e)
      {
        MsgLog.log("getDeviceNumber",e.toString(),TErrorList.code_failure,e,0);                          
        return -1; 
      }
    }
    TDevice td;
    if ((td=(TDevice)myDeviceData.get(devName.toUpperCase())) == null) return -1;
    return td.devNumber;
  }
  public int getNumberOfAlarms()
  {
    TDevice[] dlst = getDeviceList();
    if (dlst == null) return 0;
    int n = 0;
    for (int i=0; i<dlst.length; i++) n += dlst[i].getNumberOfAlarms();
    return n;
  }
  public int getNumberActiveAlarms()
  {
    TDevice[] dlst = getDeviceList();
    if (dlst == null) return 0;
    int n = 0;
    for (int i=0; i<dlst.length; i++) n += dlst[i].getNumberActiveAlarms();
    return n;    
  }
  public int getMostRecentAlarmTimestamp(int[] nalarms,int index)
  {
    TDevice[] dlst = getDeviceList();
    if (dlst == null) return 0;
    int mrt = 0, ts;
    int n = 0;
    for (int i=0; i<dlst.length; i++)
    {
      if ((ts=dlst[i].getMostRecentTimestamp()) > mrt) 
      {
        mrt = ts;
        n = 0;
      }
      if (ts == mrt) n += dlst[i].getNumberOfAlarmsAtMostRecentTimestamp();
    }
    if (nalarms != null && nalarms.length > index) nalarms[index] = n;
    return mrt;    
  }
  public int getHighestAlarmSeverity(int[] nalarms,int index)
  {
    TDevice[] dlst = getDeviceList();
    if (dlst == null) return 0;
    int hsv = 0, sv;
    int n = 0;
    for (int i=0; i<dlst.length; i++)
    {
      if ((sv=dlst[i].getHighestAlarmSeverity()) > hsv) 
      {
        hsv = sv;
        n = 0;
      }
      if (sv == hsv) n += dlst[i].getNumberOfAlarmsAtHighestSeverity();
    }
    if (nalarms != null && nalarms.length > index) nalarms[index] = n;
    return hsv;    
  }
}
